import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;

public class Hacker {

  //Map of the dictionary word
  Map<Integer, ArrayList<String>> dictionary;

  //Standard Hacker constructor using a path
  public Hacker(String dictionaryPath) {

    //Create a map of the words sorted by length of the word
    dictionary = generateDictionaryFromFile(dictionaryPath);
  }

  //Generate the map from a dictionary file
  private Map<Integer, ArrayList<String>> generateDictionaryFromFile(String path) {

    //Create the empty Map
    Map<Integer, ArrayList<String>> map = new HashMap();

    //Try to read from the file
    try {

      //Create the BufferedReader
      BufferedReader reader = new BufferedReader(new FileReader(new File(path)));
      String word = "";

      //Loop through the dictionary
      while((word = reader.readLine()) != null) {
        if (map.containsKey(word.length())) {

          //Get the already existing list and add value to it
          ArrayList<String> words = map.get(word.length());
          words.add(word);
        }
        else {

          //Create the ArrayList for that key
          ArrayList<String> words = new ArrayList<String>();
          words.add(word);

          //Add the list to the map with the word length as key
          map.put(word.length(), words);
        }
      }
    }
    catch(IOException e) {
      e.printStackTrace();
      System.out.println("Error: Could not open the dictionary file");
    }

    return map;
  }

  //Generate the wordList from the dictionary
  private ArrayList<String> generateWords(int wordLength, int numOfWords) {

    ArrayList<String> allWords = dictionary.get(wordLength);
    ArrayList<String> wordList = new ArrayList<String>();
    ArrayList<Integer> usedIndexes = new ArrayList<Integer>();

    //Add the words to list
    for (int i = 0; i < numOfWords; i++) {

      //Randomise index
      int index;
      do {
        index = (int)Math.floor(Math.random() * allWords.size());
      }
      while (usedIndexes.contains(index));

      //Add word to wordList
      wordList.add(allWords.get(index));
    }

    return wordList;
  }

  //This is the main gameplay loop
  public void run() {
    try {
      BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
      String line;
      do {
        playGame();
        System.out.println("\nWould you like to play again? [y/n]");
      }
      while((line = input.readLine().toLowerCase()).equals("y"));
    }
    catch(IOException e) {
      e.printStackTrace();
      System.out.println("Error: Coult not get the user input");
    }
  }

  //This function will start the gameplay loop
  private void playGame() {
    try {

      //BufferedReader for input
      BufferedReader input = new BufferedReader(new InputStreamReader(System.in));

      //Get difficulty
      System.out.println("What difficulty of game do you want? [1/2/3]");
      int difficulty = Integer.parseInt(input.readLine());

      //Make a list of words based off the difficulty
      ArrayList<String> wordList;
      int wordLength;
      int numOfWords;
      switch(difficulty) {
        case 1:
          wordLength = (int)Math.floor(Math.random() * 2) + 4;
          numOfWords = (int)Math.floor(Math.random() * 2) + 8;
          wordList = generateWords(wordLength, numOfWords);
          break;
        case 2:
          wordLength = (int)Math.floor(Math.random() * 2) + 6;
          numOfWords = (int)Math.floor(Math.random() * 2) + 10;
          wordList = generateWords(wordLength, numOfWords);
          break;
        case 3:
          wordLength = (int)Math.floor(Math.random() * 2) + 8;
          numOfWords = (int)Math.floor(Math.random() * 2) + 12;
          wordList = generateWords(wordLength, numOfWords);
          break;
        default:
          wordLength = (int)Math.floor(Math.random() * 2) + 6;
          numOfWords = (int)Math.floor(Math.random() * 2) + 10;
          wordList = generateWords(wordLength, numOfWords);
      }

      //Print out the words
      for(int i = 0; i < wordList.size(); i++) {
        System.out.println(wordList.get(i).toUpperCase());
      }

      //Start of gameplay loop
      int lives = 4;
      String answer = wordList.get((int)Math.floor(Math.random() * wordList.size()));
      do {
        System.out.print("\nEnter your guess: ");
        String guess = input.readLine().toLowerCase();
        while(!wordList.contains(guess)) {
          System.out.println("The word you entered is not on the list, Try again!");
          guess = input.readLine().toLowerCase();
        }

        int lettersCorrect = 0;

        //Loop through the letter and count if they have gotten it right
        for (int i = 0; i < answer.length(); i++) {
          char letter1 = Character.toLowerCase(answer.charAt(i));
          char letter2 = Character.toLowerCase(guess.charAt(i));
          if (letter1 == letter2) {
            lettersCorrect++;
          }
        }

        //Check if they have won
        System.out.println(lettersCorrect + " letters correct");
        if (lettersCorrect != answer.length()) {
          lives--;
          System.out.println(lives + " lives left!");
        }
        else {
          System.out.println("Congratulations you have guessed correctly!");
          break;
        }
      }
      while (lives > 0);
      if (lives == 0) {
        System.out.println("You lost! Scrub!");
      }
    }
    catch(IOException e) {
      e.printStackTrace();
      System.out.println("Error: Could not instantiate System.in");
    }
  }
}
